<?php
namespace App\Factory;
use Doctrine\DBAL\Connection;

class Branch {

  private $connection;

  public function __construct(Connection $connection) {
      $this->connection = $connection;
  }

  public function getUnits($search) {
    $sql = "
      SELECT CDFILIAL AS CDFILIALGRID, NMFILIAL,
        CDFILIAL || CDINSCMUNI AS CDFILIAL        
      FROM FILIAL
        WHERE
          CDFILIAL LIKE '%' || :SEARCH || '%'
          OR UPPER(NMFILIAL) LIKE '%' || :SEARCH || '%'
          OR 'T' = :SEARCH
        ORDER BY CDFILIAL
    ";

    $stmt = $this->connection->prepare($sql);
		$stmt->bindValue("SEARCH", $search);
    $result = $stmt->executeQuery();
    return $result->fetchAllAssociative();
  }

  public function getBlockedStores($params) {
    $sql = "
      SELECT LB.CDLOJA AS CDFILIALGRID, LB.CDFILIAL||LB.CDLOJA AS CDFILIAL,
        LO.NMLOJA, OP.NMOPERADOR,
        TO_CHAR(LB.DTHRBLOQUEIO, 'DD/MM/YYYY HH24:MI') AS DTHRBLOQUEIO,
        NVL(LB.NRDIASEMANABLOQ, 'T') AS NRDIASEMANABLOQ
      FROM LOJABLOQ LB
        JOIN LOJA LO
          ON LB.CDFILIAL = LO.CDFILIAL
          AND LB.CDLOJA   = LO.CDLOJA

        JOIN OPERADOR OP
          ON LB.CDOPERADOR = OP.CDOPERADOR

      WHERE LB.CDFILIAL = :CDFILIAL
        ORDER BY LB.CDLOJA
  ";

    $stmt = $this->connection->prepare($sql);
    $stmt->bindValue("CDFILIAL", $params['CDFILIAL']);
    $result = $stmt->executeQuery();
    return $result->fetchAllAssociative();
  }

  public function getAvailableStoresByUnit($params) {
    $sql = "
      SELECT LO.CDFILIAL, LO.CDLOJA, LO.NMLOJA
        FROM LOJA LO
      LEFT JOIN LOJABLOQ LB
        ON LB.CDFILIAL  = LO.CDFILIAL
        AND LB.CDLOJA   = LO.CDLOJA
      WHERE LO.CDFILIAL = :CDFILIAL
        AND LB.CDLOJA IS NULL
      ORDER BY LO.CDLOJA
    ";

    $stmt = $this->connection->prepare($sql);
    $stmt->bindValue("CDFILIAL", $params['CDFILIAL']);
    $result = $stmt->executeQuery();
    return $result->fetchAllAssociative();
  }

  public function blockStores($params) {    
    $sql = "
      INSERT INTO LOJABLOQ
        (CDFILIAL, CDLOJA, CDOPERADOR, DTHRBLOQUEIO, NRDIASEMANABLOQ)
      VALUES
        (:CDFILIAL, :CDLOJA, :CDOPERADOR, SYSDATE, :NRDIASEMANABLOQ)
    ";
    $stmt = $this->connection->prepare($sql);
    $stmt->bindValue("CDFILIAL", $params['CDFILIAL']);
    $stmt->bindValue("CDLOJA", $params['CDLOJA']);
    $stmt->bindValue("CDOPERADOR", '022721578820');
    $stmt->bindValue("NRDIASEMANABLOQ", $params['NRDIASEMANABLOQ']);
    return $stmt->executeQuery();
  }
}
