import { alertService } from '@/helpers/AlertService';
import Graph from '@/helpers/Graph';
import DateTime from '@/helpers/DateTime';
import { LoadingService } from '@zeedhi/common';
import { Http } from '@zeedhi/core';

export class DashboardController {
  public cardNumber = 13;

  public todayMonth = new Date().getMonth();

  public retroMonths = 6;

  public quantCancel: string = '0';

  public quantCancelDiff: string = '0';

  public quantSales: string = '0';

  public quantSalesDiff: string = '0';

  public quantSalesToday: string = '0';

  public valueSales: string = 'R$ 0,00';

  public valueSalesDiff: string = 'R$ 0,00';

  public valueSalesToday: string = 'R$ 0,00';

  public quantCancelToday: string = '0';

  public valueCancelToday: string = 'R$ 0,00';

  public valueCancel: string = 'R$ 0,00';

  public valueCancelDiff: string = 'R$ 0,00';

  public async createDashboard() {
    try {
      LoadingService.show();
      const today = new Date();
      const params = {
        dtInicial: this.getMonthsAgo(6),
        dtFinal: `${today.getDate()}/${today.getMonth() + 1}/${today.getFullYear()} 23:59:59`,
      };

      const { data: { data } } = await Http.post('/order/dashboard', params);

      this.createGraphSalesToday(data);
      this.createGraphSalesHour(data.salesHour);
      this.createGraphSalesSevenDays(data);
      this.createGraphSalesCurrentAndLastMonth(data);
      this.createGraphSalesMonthOrigin(data);
      this.createGraphSalesMonthModality(data);
      this.createGraphSalesMonthOriginCancel(data);
    } catch (error) {
      if (error instanceof Error) {
        alertService('error', error.message, 'red');
      }
    } finally {
      LoadingService.hide();
    }
  }

  public createGraphSalesToday(data: any) {
    try {
      const datasource = [data.seriesInnerData, data.seriesOuterData];
      const title = 'Pedidos Hoje';
      const name = 'sales today';
      const id = `dashboardCard${this.cardNumber}`;
      this.cardNumber += 1;
      const color = ['#5470c6', '#91cc75', '#fac858', '#ee6666', '#73c0de', '#3ba272', '#fc8452', '#9a60b4', '#ea7ccc'];

      const params = {
        datasource,
        title,
        name,
        id,
        color,
      };
      this.createCardValueSalesToday(data.valueSalesToday);
      this.createCardValueCancelToday(data.valueCancelToday);
      this.createCardQuantSalesToday(data.quantSalesToday);
      this.createCardQuantCancelToday(data.quantCancelToday);
      Graph.createGraphPieNest(params);
    } catch (error) {
      if (error instanceof Error) {
        alertService('error', error.message, 'red');
      }
    }
  }

  public createGraphSalesSevenDays(data: any) {
    try {
      const weekDays = 7;
      const header = ['Dia', 'Cancelados', 'Pedidos Finalizados', 'Valores'];
      const datasource = [];
      for (let countDays = 0; countDays < weekDays; countDays += 1) {
        datasource.push([
          `${data.lastWeekDatasource[countDays][0]}`,
          `${data.lastWeekDatasource[countDays][1]}`,
          `${data.lastWeekDatasource[countDays][2]}`,
          `${data.lastWeekDatasource[countDays][3]}`,
        ]);
      }
      datasource.reverse();
      datasource.unshift(header);
      const title = 'Pedidos/Valores dos Últimos 7 Dias';
      const name = 'Seven Days';
      const id = `dashboardCard${this.cardNumber}`;
      this.cardNumber += 1;
      const color = ['#e33b3b', '#e9561c', '#91cc75'];
      const tooltip = '';
      const yAxis = [
        {
          name: 'Pedidos',
          type: 'value',
        },
        {
          name: 'Valor',
          type: 'value',
          axisLabel: {
            formatter:
             (value: number | string) => `R$ ${value.toString().replace('.', ',').replace(/\B(?=(\d{3})+(?!\d))/g, '.')}`,
          },
        },
      ];

      const params = {
        datasource,
        title,
        name,
        id,
        color,
        tooltip,
        yAxis,
      };
      Graph.createGraphBar(params);
    } catch (error) {
      if (error instanceof Error) {
        alertService('error', error.message, 'red');
      }
    }
  }

  public createGraphSalesCurrentAndLastMonth(data: any) {
    try {
      const allMonthsNames = DateTime.getAllMonthNames();
      const PreviousMonthParams = {
        months: 1,
      };

      const prevMonth = DateTime.getPreviousMonths(PreviousMonthParams);
      const curMonth = allMonthsNames[this.todayMonth];

      const datasource = [
        ['product', 'Cancelados', 'Pedidos Finalizados', 'Valores'],
        [...prevMonth, data.quantCancelLastMonth, data.quantSalesLastMonth, data.valueSalesLastMonth.toFixed(2)],
        [curMonth, data.quantCancelCurrentMonth, data.quantSalesCurrentMonth, data.valueSalesCurrentMonth.toFixed(2)]];
      const title = 'Pedidos/Valores do Último Mês e do Mês Atual';
      const name = 'Current Month';
      const id = `dashboardCard${this.cardNumber}`;
      this.cardNumber += 1;
      const yAxis = [
        {
          name: 'Pedidos',
          type: 'value',
        },
        {
          name: 'Valor',
          type: 'value',
          axisLabel: {
            formatter: 'R$ {value}',
          },
        },
      ];
      const color = ['#e33b3b', '#e9561c', '#91cc75'];
      const tooltip = '';
      const params = {
        datasource,
        title,
        name,
        id,
        color,
        tooltip,
        yAxis,
      };
      Graph.createGraphBar(params);
    } catch (error) {
      if (error instanceof Error) {
        alertService('error', error.message, 'red');
      }
    }
  }

  public createGraphSalesMonthModality(data: any) {
    try {
      const title = 'Pedidos Mensais por Modalidade';
      const name = 'sales modality per month';
      const id = `dashboardCard${this.cardNumber}`;
      this.cardNumber += 1;
      const tooltip = '';

      const params = {
        datasource: data.salesMonthModalityDatasource,
        title,
        name,
        id,
        tooltip,
        legendData: data.salesMonthModalityLegend,
        data: data.salesMonthModalityData,
      };

      this.createCardQuantSales(data.quantSalesLast6Month);
      this.createCardQuantSalesDiff(data.quantSalesLastMonth, data.quantSalesCurrentMonth);
      this.createCardValueSales(data.valueSalesLast6Month);
      this.createCardValueSalesDiff(data.valueSalesLastMonth, data.valueSalesCurrentMonth);
      Graph.createGraphLine(params);
    } catch (error) {
      if (error instanceof Error) {
        alertService('error', error.message, 'red');
      }
    }
  }

  public createGraphSalesMonthOrigin(data: any) {
    try {
      const title = 'Pedidos Mensais por Origem';
      const name = 'sales origin per month';
      const id = `dashboardCard${this.cardNumber}`;
      this.cardNumber += 1;
      const tooltip = '';

      const params = {
        datasource: data.salesMonthOriginDatasource,
        title,
        name,
        id,
        tooltip,
        legendData: data.salesMonthOriginLegend,
        data: data.salesMonthOriginData,
      };
      Graph.createGraphLine(params);
    } catch (error) {
      if (error instanceof Error) {
        alertService('error', error.message, 'red');
      }
    }
  }

  public createGraphSalesMonthOriginCancel(data: any) {
    try {
      const title = 'Cancelamentos Mensais por Origem';
      const name = 'cancel origin per month';
      const id = `dashboardCard${this.cardNumber}`;
      this.cardNumber += 1;
      const tooltip = '';

      const params = {
        datasource: data.cancelMonthOriginDatasource,
        title,
        name,
        id,
        tooltip,
        legendData: data.cancelMonthOriginLegend,
        data: data.cancelMonthOriginData,
      };

      this.createCardQuantCancel(data.quantCancelLast6Month);
      this.createCardQuantCancelDiff(data.quantCancelLastMonth, data.quantCancelCurrentMonth);
      this.createCardValueCancel(data.valueCancelLast6Month);
      this.createCardValueCancelDiff(data.valueCancelLastMonth, data.valueCancelCurrentMonth);
      Graph.createGraphLine(params);
    } catch (error) {
      if (error instanceof Error) {
        alertService('error', error.message, 'red');
      }
    }
  }

  public createGraphSalesDelay(dashboard: any) {
    try {
      const PreviousMonthParams = {
        months: this.retroMonths,
        includeCurrentMonth: true,
      };
      const data = DateTime.getPreviousMonths(PreviousMonthParams);
      const datasource = [{
        name: 'Pedidos Atrasados',
        type: 'line',
        data: this.createArray(12),
        label: {
          show: true,
          position: 'top',
          color: 'black',
          fontSize: 12,
          formatter(d: { data: string; }) {
            return `${d.data}`;
          },
        },
      }];

      const salesDelayed = dashboard.reduce((acc: any, cv: any) => {
        const [day, month, year] = this.getDayMonthYear(cv.DTHRABERMESA);
        const timeHour = cv.DTHRABERMESA.split(' ');
        const endSale = cv.DTHRFECHMESA || new Date();
        const timeExpected = new Date(`${month}/${day}/${year} ${timeHour[1]}`);
        timeExpected.setMinutes(timeExpected.getMinutes() + parseInt(cv.TEMPOPEDIDO, 10));
        const intMonth = parseInt(month, 10) - 1;
        acc[intMonth] = acc[intMonth] || {};
        if (endSale > timeExpected && cv.IDSTCOMANDA < 4) {
          acc[intMonth] = parseInt(acc[intMonth], 10) + 1 || 1;
        }
        return acc;
      }, {});
      const nextMonth = new Date();
      nextMonth.setMonth(nextMonth.getMonth() - this.retroMonths);
      let k = 0;
      for (nextMonth; nextMonth.getMonth() !== this.todayMonth + 1; nextMonth.setMonth(nextMonth.getMonth() + 1), k += 1) {
        datasource[0].data[k] = salesDelayed[nextMonth.getMonth()] || 0;
        data[k] = data[k].concat(`(${datasource[0].data[k]})`);
      }

      const title = 'Atrasos Mensais';
      const name = 'delay monthly';
      const id = `dashboardCard${this.cardNumber}`;
      this.cardNumber += 1;
      const tooltip = '';
      const legendData = ['Pedidos Atrasados'];

      const params = {
        datasource,
        title,
        name,
        id,
        tooltip,
        legendData,
        data,
      };

      Graph.createGraphLine(params);
    } catch (error) {
      if (error instanceof Error) {
        alertService('error', error.message, 'red');
      }
    }
  }

  public createGraphSalesHour(salesHour: any[][]) {
    try {
      const data = ['0:00', '1:00', '2:00', '3:00', '4:00', '5:00', '6:00', '7:00', '8:00', '9:00', '10:00', '11:00',
        '12:00', '13:00', '14:00', '15:00', '16:00', '17:00', '18:00', '19:00', '20:00', '21:00', '22:00', '23:00'];
      const datasource = [{
        name: 'Pedidos',
        type: 'line',
        data: [this.createArray(24)],
        label: {
          show: true,
          position: 'top',
          color: 'black',
          fontSize: 12,
          formatter(d: { data: string; }) {
            return `${d.data}`;
          },
        },
      }];

      datasource[0].data = salesHour;
      const title = 'Horário dos Pedidos';
      const name = 'sales per hour';
      const id = `dashboardCard${this.cardNumber}`;
      this.cardNumber += 1;
      const color = ['#e9561c'];
      const tooltip = '';
      const legendData = ['Pedidos'];

      const params = {
        datasource,
        title,
        name,
        id,
        color,
        tooltip,
        legendData,
        data,
      };
      Graph.createGraphLine(params);
    } catch (error) {
      if (error instanceof Error) {
        alertService('error', error.message, 'red');
      }
    }
  }

  public createGraphSalesDelayHour(dashboard: any) {
    try {
      const data = ['0:00', '1:00', '2:00', '3:00', '4:00', '5:00', '6:00', '7:00', '8:00', '9:00', '10:00', '11:00',
        '12:00', '13:00', '14:00', '15:00', '16:00', '17:00', '18:00', '19:00', '20:00', '21:00', '22:00', '23:00'];
      const datasource = [{
        name: 'Pedidos Atrasados',
        type: 'line',
        data: this.createArray(24),
        label: {
          show: true,
          position: 'top',
          color: 'black',
          fontSize: 12,
          formatter(d: { data: string; }) {
            return `${d.data}`;
          },
        },
      }];

      dashboard.forEach((element: any) => {
        const [day, month, year] = this.getDayMonthYear(element.DTHRABERMESA);
        const timeHour = element.DTHRABERMESA.split(' ');
        const partHour = timeHour[1].split(':');
        const hour = parseInt(partHour[0], 10);
        const minute = parseInt(partHour[1], 10);
        const second = parseInt(partHour[2], 10);
        const endSale = element.DTHRFECHMESA || new Date();
        const timeExpected = new Date(`${month}/${day}/${year} ${hour}:${minute}:${second}`);
        timeExpected.setMinutes(timeExpected.getMinutes() + element.TEMPOPEDIDO);
        if (endSale > timeExpected && element.IDSTCOMANDA < 4) {
          datasource[0].data[hour] += 1;
        }
      });

      const title = 'Horário dos Pedidos Atrasados';
      const name = 'cancel per hour';
      const id = `dashboardCard${this.cardNumber}`;
      this.cardNumber += 1;
      const tooltip = '';
      const legendData = ['Pedidos Atrasados'];

      const params = {
        datasource,
        title,
        name,
        id,
        tooltip,
        legendData,
        data,
      };

      Graph.createGraphLine(params);
    } catch (error) {
      if (error instanceof Error) {
        alertService('error', error.message, 'red');
      }
    }
  }

  public createCardQuantSales(quant: number) {
    const formattedValue = quant.toString().replace(/\B(?=(\d{3})+(?!\d))/g, '.');
    this.quantSales = `${formattedValue}`;
  }

  public createCardValueSales(quant: number) {
    const formattedValue = quant.toFixed(2).replace('.', ',');
    const [integerPart, decimalPart] = formattedValue.split(',');
    const integerPartWithThousand = integerPart.replace(/\B(?=(\d{3})+(?!\d))/g, '.');
    this.valueSales = `R$ ${integerPartWithThousand},${decimalPart}`;
  }

  public createCardQuantCancel(quant: number) {
    const formattedValue = quant.toString().replace(/\B(?=(\d{3})+(?!\d))/g, '.');
    this.quantCancel = `${formattedValue}`;
  }

  public createCardQuantCancelDiff(lastMonth: number, currMonth: number) {
    const formattedValue = (currMonth - lastMonth).toString().replace(/\B(?=(\d{3})+(?!\d))/g, '.');
    this.quantCancelDiff = `${formattedValue}`;
  }

  public createCardQuantSalesDiff(lastMonth: number, currMonth: number) {
    const formattedValue = (currMonth - lastMonth).toString().replace(/\B(?=(\d{3})+(?!\d))/g, '.');
    this.quantSalesDiff = `${formattedValue}`;
  }

  public createCardValueSalesDiff(lastMonth: number, currMonth: number) {
    const formattedValue = (currMonth - lastMonth).toFixed(2).replace('.', ',');
    const [integerPart, decimalPart] = formattedValue.split(',');
    const integerPartWithThousand = integerPart.replace(/\B(?=(\d{3})+(?!\d))/g, '.');
    this.valueSalesDiff = `R$ ${integerPartWithThousand},${decimalPart}`;
  }

  public createCardQuantSalesToday(quant: number) {
    const formattedValue = quant.toString().replace(/\B(?=(\d{3})+(?!\d))/g, '.');
    this.quantSalesToday = `<strong>${formattedValue}</strong>`;
  }

  public createCardValueSalesToday(quant: number) {
    const formattedValue = quant.toFixed(2).replace('.', ',');
    const [integerPart, decimalPart] = formattedValue.split(',');
    const integerPartWithThousand = integerPart.replace(/\B(?=(\d{3})+(?!\d))/g, '.');
    this.valueSalesToday = `R$ ${integerPartWithThousand},${decimalPart}`;
  }

  public createCardQuantCancelToday(quant: number) {
    const formattedValue = quant.toString().replace(/\B(?=(\d{3})+(?!\d))/g, '.');
    this.quantCancelToday = `${formattedValue}`;
  }

  public createCardValueCancel(quant: number) {
    const formattedValue = quant.toFixed(2).replace('.', ',');
    const [integerPart, decimalPart] = formattedValue.split(',');
    const integerPartWithThousand = integerPart.replace(/\B(?=(\d{3})+(?!\d))/g, '.');
    this.valueCancel = `R$ ${integerPartWithThousand},${decimalPart}`;
  }

  public createCardValueCancelDiff(lastMonth: number, currMonth: number) {
    const formattedValue = (currMonth - lastMonth).toFixed(2).replace('.', ',');
    const [integerPart, decimalPart] = formattedValue.split(',');
    const integerPartWithThousand = integerPart.replace(/\B(?=(\d{3})+(?!\d))/g, '.');
    this.valueCancelDiff = `R$ ${integerPartWithThousand},${decimalPart}`;
  }

  public createCardValueCancelToday(quant: number) {
    const formattedValue = quant.toFixed(2).replace('.', ',');
    const [integerPart, decimalPart] = formattedValue.split(',');
    const integerPartWithThousand = integerPart.replace(/\B(?=(\d{3})+(?!\d))/g, '.');
    this.valueCancelToday = `R$ ${integerPartWithThousand},${decimalPart}`;
  }

  private getDayMonthYear(DTHRABERMESA: string) {
    let [day, month, year] = DTHRABERMESA.split('/');
    day = day.replace(/^0+/, '');
    month = month.replace(/^0+/, '');
    year = year.replace(/^0+/, '');
    return [day, month, year.slice(0, 4)];
  }

  private createArray(size: number, defaultValue = 0) {
    return [...Array(size)].fill(defaultValue, 0, size);
  }

  private getMonthsAgo(monthsAgo: number): string {
    const monthAgo = new Date();
    monthAgo.setMonth(monthAgo.getMonth() - monthsAgo);
    const month = `${monthAgo.getMonth() + 1}`;

    const date = `01/${month.padStart(2, '0')}/${monthAgo.getFullYear()} 00:00:00`;

    return date;
  }
}
