/* eslint-disable */
import { Form, Loading, LoadingService, Modal, ModalService } from '@zeedhi/common';
import { TekGrid } from '@zeedhi/teknisa-components-common';
import { Http, IEventParam, Metadata } from '@zeedhi/core';
import { Instances } from '@/types/InterfaceStoreBlockController';
import { ZdSelect } from '@zeedhi/vuetify';
import { alertService } from '@/helpers/AlertService';

interface RowUnitBlock {
  CDFILIAL: string,
  CDLOJA: string,
  NMLOJA: string,
}

interface Row {
  CDFILIALGRID: string,
  CDFILIAL: string,
  NMLOJA: string,
  NMOPERADOR: string,
  DTHRBLOQUEIO: string,
  NRDIASEMANABLOQ: string,
  PARENTID: string,
  LABELNRDIASEMANA: string,
}
interface ParamsBlockStore {
  CDFILIAL: string,
  CDLOJA: string[],
  CDOPERADOR: string | null,
  NRDIASEMANABLOQ: string[] | string,
}
export class StoreBlockController {
  private storeBlockModal!: Modal;

  public async createStoreBlockModal() {
    this.storeBlockModal = await ModalService.createFromJson('store-block-modal', true);
  }

  public destroyStoreBlockModal() {
    this.storeBlockModal?.destroy();
  }

  public openStoreBlockModal() {
    this.storeBlockModal.show();
  }

  public closeStoreBlockModal() {
    this.storeBlockModal.hide();
  }

  public grid(): TekGrid {
    return Metadata.getInstance<TekGrid>('store-block-grid');
  }

  public async openStoreModal({ row }: IEventParam<TekGrid>): Promise<void> {
    if (row.PARENTID) return;
    if (this.storeBlockModal) this.destroyStoreBlockModal();
    this.storeBlockModal = await ModalService.createFromJson('store-block-modal', true);
    this.getUnit(row);
  }

  public async getUnit({ CDFILIALGRID }: Row): Promise<void> {
    try {
      this.openStoreBlockModal();
      const form = await this.getInstance<Form>('store-block-form');
      form.value = { CDFILIALGRID };

      const select = await this.getSelectStore();
      const storeData = await this.fetchStoreData(CDFILIALGRID);
      if (storeData.length > 0) select.datasource.data?.push(...storeData);
    } catch (error) {
      if (error instanceof Error) {
        alertService('error', error.message, 'red');
      }
    }
  }

  public async fetchStoreData(CDFILIAL: string): Promise<any> {
    try {
      const { data: { data, error, message } } =
        await Http.post('/branch/availableStoresRepository', { CDFILIAL });

      if (error) throw new Error(message || 'Erro na solicitação da API');
      if (data) return data || [];
    } catch (error) {
      if (error instanceof Error) {
        throw new Error(error.message);
      }
    }
  } 

  public async getInstance<T extends Instances>(name: string): Promise<T> {
    const instance: T = await new Promise((resolve) => {
      setTimeout(() => {
        resolve(Metadata.getInstances(name)[0]);
      }, 0);
    });
    return instance;
  }

  public async getSelectStore(): Promise<ZdSelect> {
    const form = await this.getInstance<Form>('store-block-form');
    const select = form.getChildInstance<ZdSelect>('store');
    return select;
  }

  public async isValidFormBlockStore(): Promise<boolean> {
    const form = await this.getInstance<Form>('store-block-form');
    const valid = form.validate();
    return valid;
  }

  public async paramsBlockStoreModal(): Promise<ParamsBlockStore> {
    const form = await this.getInstance<Form>('store-block-form');
    const { value } = form;
    const cdOperador = localStorage.getItem('ID');
    return {
      CDFILIAL: value.CDFILIALGRID,
      CDLOJA: value.store,
      CDOPERADOR: cdOperador,
      NRDIASEMANABLOQ: value.week.length === 7 ? 'T' : value.week,
    };
  }

  public async blockStore(): Promise<any> {
    try {
      const isValid = await this.isValidFormBlockStore();

      if (!isValid) return;
      const params = await this.paramsBlockStoreModal();
      const grid = this.grid();

      LoadingService.show();
      const { data: { error, message } } = await Http.post('branch/blockStores', params);

      if (error) throw new Error(message || 'Erro na solicitação da API');

      alertService('success', message, 'green');
      grid.reload();
    } catch (error) {
      if (error instanceof Error) {
        alertService('error', error.message, 'red');
      }
    } finally {
      this.destroyStoreBlockModal();
      LoadingService.hide();
    }
  }
}